import { Message } from '../core/type';
import { Codec, CodecSelector, defaultCodec } from './codec';

const { gzip, ungzip } = require('pako');

export class GZipCodec implements Codec {
  public encode(message: string): Message {
    return gzip(message).buffer as ArrayBuffer;
  }
  public decode(message: Message): string {
    return ungzip(Buffer.from(message as ArrayBuffer), { to: 'string' });
  }
}

export const gZipCodec = new GZipCodec();

export class CompressionCodecSelector implements CodecSelector {
  private readonly threshold: number;

  public constructor(threshold: number) {
    this.threshold = threshold;
  }

  public encode(message: string): Message {
    if (this.threshold < Buffer.byteLength(message)) {
      return gZipCodec.encode(message);
    } else {
      return defaultCodec.encode(message);
    }
  }

  public decode(message: Message): string {
    if (message instanceof ArrayBuffer) {
      return gZipCodec.decode(message);
    } else {
      return defaultCodec.decode(message);
    }
  }
}
