import { CodecSelector, defaultCodecSelector } from '../../codec/codec';
import { Message } from '../../core/type';
import { ClientCodec, ClientResponseCodec } from '../client_codec';
import { ClientRequest } from './client_request';
import { JSONClientResponseCodec } from './client_response';
import { version } from './constants';
import { convertParamsToStringArray } from './util';

export class JSONClientCodec implements ClientCodec {
  private readonly codecSelector: CodecSelector;

  public constructor(codecSelector: CodecSelector = defaultCodecSelector) {
    this.codecSelector = codecSelector;
  }

  public request(method: string, args: any[] | null, id?: number): Message {
    const params = convertParamsToStringArray(args);

    const req: ClientRequest = {
      jsonrpc: version,
      method,
      params: 0 === params.length ? null : params,
      id: undefined !== id && 0 < id ? id : undefined,
    };

    return this.codecSelector.encode(JSON.stringify(req));
  }

  public response(message: Message): ClientResponseCodec {
    return new JSONClientResponseCodec(this.codecSelector.decode(message));
  }
}
