import { ClientNotificationCodec, ClientResponseCodec } from '../client_codec';
import { RPCError } from '../error';
import {
  ClientNotification,
  JSONClientNotificationCodec,
} from './client_notification';

export interface ClientResponse {
  jsonrpc: string;
  result?: any;
  error?: RPCError;
  id?: number;
}

export class JSONClientResponseCodec implements ClientResponseCodec {
  private res: ClientResponse;

  public constructor(json: string) {
    this.res = JSON.parse(json);
  }

  public id(): number | undefined {
    return this.res.id;
  }
  public error(): RPCError | undefined {
    return this.res.error;
  }
  public result(): any | undefined {
    return this.res.result;
  }

  public isNotification(): boolean {
    if (undefined === this.id() && undefined !== this.result()) {
      return true;
    }

    return false;
  }

  public notification(): ClientNotificationCodec | undefined {
    if (undefined !== this.id() || undefined === this.result()) {
      return undefined;
    }
    const noti: ClientNotification = {
      method: this.res.result.method,
      params: this.res.result.params,
    };

    return new JSONClientNotificationCodec(noti);
  }
}
